/***********************************************************
 * $Id: fancypcsccard.h 1243 2014-05-29 20:49:19Z fancylibs $
 *
 * FancyLibs PC/SC wrapper
 *
 * Copyright (C) 2014 ITEG IT-Engineers GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************/

#ifndef FANCYPCSCCARD_H_
#define FANCYPCSCCARD_H_

#include <fancypcsc.h>
#include <fancyobject.h>

#include <string>

namespace Fancy {

template<typename T> class ArrayObject;
template<typename T> class Array;

namespace PCSC {

class FANCYPCSC_API PCSCCard : virtual public Fancy::Object {

protected:
	PCSCCard();
	virtual ~PCSCCard();

public:

	typedef Fancy::ObjPtr<PCSCCard> ptr_t;
	/**
	 * @return The selected protocol or PROTOCOL_UNNKNOWN, if no card is present or we
	 *         have not yet connected to a reader.
	 */
	virtual int getProtocol() = 0;
	virtual Fancy::ObjPtr<ArrayObject<int8_t> > transmit(const Fancy::Array<int8_t> &bytes, size_t nbytes) = 0;
	virtual Fancy::ObjPtr<ArrayObject<int8_t> > transmit(const Fancy::ObjPtr<ArrayObject<int8_t> > &bytes) = 0;
	virtual Fancy::ObjPtr<ArrayObject<int8_t> > transmit(const int8_t *bytes, size_t nbytes) = 0;

	virtual void unpower() = 0;
	virtual void reset() = 0;
	virtual void eject() = 0;
	virtual void disconnect() = 0;

	static std::string getNameOfProtocol(int protocol);
};

} /* namespace PCSC */
} /* namespace Fancy */

#endif /* FANCYPCSCCARD_H_ */
